@@ -1,5 +1,5 @@
 ;;; emacspeak-ediff.el --- Speech enable Emacs interface to diff and merge
-;;; $Id: emacspeak-ediff.el,v 16.0 2002/05/03 23:31:23 raman Exp $
+;;; $Id: emacspeak-ediff.el,v 20.0 2004/05/01 01:16:22 raman Exp $
 ;;; $Author: raman $ 
 ;;; DescriptionEmacspeak extensions for ediff
 ;;; Keywords:emacspeak, audio interface to emacs, Comparing files 
@@ -8,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@cs.cornell.edu 
 ;;; A speech interface to Emacs |
-;;; $Date: 2002/05/03 23:31:23 $ |
-;;;  $Revision: 16.0 $ | 
+;;; $Date: 2004/05/01 01:16:22 $ |
+;;;  $Revision: 20.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2002, T. V. Raman 
+;;;Copyright (C) 1995 -- 2003, T. V. Raman 
 ;;; Copyright (c) 1995 by .
 ;;; All Rights Reserved. 
 ;;;
@@ -39,19 +39,7 @@
 
 ;;{{{  required 
 
-(eval-when-compile (require 'cl))
-(declaim  (optimize  (safety 0) (speed 3)))
-(require 'backquote)
-(require 'custom)
-(eval-when-compile (require 'dtk-speak)
-(require 'emacspeak-speak)
-(require 'emacspeak-keymap)
-(require 'emacspeak-sounds)
-(require 'voice-lock)
-        (require 'ediff)
-        (require 'ediff-init))
-    
-
+(require 'emacspeak-preamble)
 ;;}}}
 ;;{{{  Introduction:
 
@@ -68,7 +56,7 @@
 ;;}}}
 ;;{{{  macros
 
-(defgroup  emacspeak-ediff nil
+(defgroup emacspeak-ediff nil
   "Emacspeak support for EDiff."
   :link '(custom-group-link :tag "ediff"
                             ediff)
@@ -88,32 +76,30 @@
        (setq buffer-read-only save-read-only )
        (set-buffer-modified-p modification-flag )))))
 
-
 ;;}}}
 ;;{{{  Mapping faces to personalities:
 
-(defcustom emacspeak-ediff-A-personality 'paul-smooth
+(defcustom emacspeak-ediff-A-personality voice-smoothen
   "Personality used to voiceify difference chunk A"
   :type 'symbol
   :group 'emacspeak-ediff)
 
 (defcustom emacspeak-ediff-B-personality
-  'paul-monotone
+  voice-monotone
   "Personality used to voiceify difference chunk B"
   :type 'symbol
-:group 'emacspeak-ediff)
+  :group 'emacspeak-ediff)
 
-(defcustom emacspeak-ediff-fine-A-personality 'harry
+(defcustom emacspeak-ediff-fine-A-personality voice-bolden
   "Personality used to voiceify difference chunk A"
   :type 'symbol
   :group 'emacspeak-ediff)
 
 (defcustom emacspeak-ediff-fine-B-personality
-  'harry
+  voice-bolden
   "Personality used to voiceify difference chunk B"
   :type 'symbol
-:group 'emacspeak-ediff)
-
+  :group 'emacspeak-ediff)
 
 ;;}}}
 ;;{{{ Helper functions:
@@ -126,7 +112,6 @@
   (declare (special emacspeak-ediff-control-buffer))
   (setq emacspeak-ediff-control-buffer (ad-get-arg 0 )))
 
-
 (defsubst emacspeak-ediff-control-panel ()
   (declare (special emacspeak-ediff-control-buffer ))
   emacspeak-ediff-control-buffer)
@@ -149,13 +134,13 @@
 
 (defsubst emacspeak-ediff-difference-c-overlay (n)
   (declare (special ediff-difference-vector-B
+                    ediff-difference-vector-C
                     ediff-number-of-differences))
   (assert (< n ediff-number-of-differences) t
           "There are only %s differences"
           ediff-number-of-differences)
   (aref (aref ediff-difference-vector-C n) 0))
 
-
 (defsubst emacspeak-ediff-fine-difference-a-overlays (n)
   (declare (special ediff-difference-vector-A
                     ediff-number-of-differences))
@@ -174,6 +159,7 @@
 
 (defsubst emacspeak-ediff-fine-difference-c-overlays (n)
   (declare (special ediff-difference-vector-B
+                    ediff-difference-vector-C
                     ediff-number-of-differences))
   (assert (< n ediff-number-of-differences) t
           "There are only %s differences"
@@ -189,7 +175,6 @@
 (defsubst  emacspeak-ediff-diff-overlay-from-difference  (diff counter )
   (aref (aref diff counter) 0))
 
-
 (defsubst emacspeak-ediff-fine-overlays-from-difference  (diff counter )
   (aref (aref diff counter) 1))
 
@@ -267,6 +252,7 @@
   "Voicify all the difference chunks"
   (declare (special ediff-buffer-A ediff-buffer-B
                     ediff-number-of-differences
+		    ediff-difference-vector-B ediff-difference-vector-A
                     emacspeak-ediff-A-personality
                     emacspeak-ediff-B-personality
                     emacspeak-ediff-fine-A-personality
@@ -341,7 +327,6 @@
 ;;; If you see keyboard activity, shut up
 ;;; and offer to speak difference b.
 
-
 (defun emacspeak-ediff-speak-difference (n)
   "Speak a difference chunk"
   (let ((a-overlay (emacspeak-ediff-difference-a-overlay n ))
@@ -484,8 +469,6 @@ Set this to nil if things get too slow."
     (message "turned %s file name truncation in Ediff registry"
              ediff-meta-truncate-filenames)))
 
-
-
 ;;}}}
 (provide  'emacspeak-ediff)
 ;;{{{  emacs local variables 
@@ -495,5 +478,4 @@ Set this to nil if things get too slow."
 ;;; byte-compile-dynamic: t
 ;;; end: 
 
-
 ;;}}}
