@@ -1,5 +1,5 @@
 ;;; emacspeak-pronounce.el --- Implements Emacspeak pronunciation dictionaries
-;;; $Id: emacspeak-pronounce.el,v 16.0 2002/05/03 23:31:23 raman Exp $
+;;; $Id: emacspeak-pronounce.el,v 20.0 2004/05/01 01:16:23 raman Exp $
 ;;; $Author: raman $
 ;;; Description: Emacspeak pronunciation dictionaries
 ;;; Keywords:emacspeak, audio interface to emacs customized pronunciation
@@ -8,14 +8,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@cs.cornell.edu
 ;;; A speech interface to Emacs |
-;;; $Date: 2002/05/03 23:31:23 $ |
-;;;  $Revision: 16.0 $ |
+;;; $Date: 2004/05/01 01:16:23 $ |
+;;;  $Revision: 20.0 $ |
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2002, T. V. Raman 
+;;;Copyright (C) 1995 -- 2003, T. V. Raman 
 ;;; Copyright (c) 1995 by T. V. Raman 
 ;;; All Rights Reserved.
 ;;;
@@ -61,11 +61,12 @@
 ;;; Code:
 ;;{{{ required packages
 
-(eval-when-compile (require 'cl))
+(require 'cl)
 (declaim  (optimize  (safety 0) (speed 3)))
 (require 'custom)
 (eval-when-compile (require 'wid-edit)
- (require 'voice-lock))
+		   (require 'emacspeak-personality))
+(require 'voice-setup)
 (require 'thingatpt)
 (eval-when (compile)
                                         ;avoid recursive include during compile
@@ -73,6 +74,13 @@
   (require 'dtk-speak)
   (require 'emacspeak-sounds))
 
+;;}}}
+;;{{{ customizations 
+
+(defgroup emacspeak-pronounce nil
+  "Pronunciation dictionaries."
+  :group 'emacspeak)
+
 ;;}}}
 ;;{{{  Dictionary structure:
 
@@ -91,8 +99,8 @@ Values are alists containing string.pronunciation pairs.")
   (declare (special emacspeak-pronounce-dictionaries))
   (when (stringp key)
     (setq key (intern key )))
-  (cl-gethash key emacspeak-pronounce-dictionaries))
-
+  (gethash key emacspeak-pronounce-dictionaries))
+;;;###autoload
 (defun emacspeak-pronounce-add-dictionary-entry  (key string pronunciation)
   "Add dictionary entry.
 This adds pronunciation pair
@@ -108,7 +116,14 @@ Argument KEY specifies a dictionary key e.g. directory, mode etc."
      (t (emacspeak-pronounce-set-dictionary key
                                             (list (cons string
                                                         pronunciation)))))))
+(defun emacspeak-pronounce-remove-buffer-local-dictionary-entry (string)
+  "Remove pronunciation entry."
+  (when (and (boundp 'emacspeak-pronounce-pronunciation-table)
+             emacspeak-pronounce-pronunciation-table)
+    (remhash    string
+                emacspeak-pronounce-pronunciation-table)))
 
+  
 (defun emacspeak-pronounce-add-buffer-local-dictionary-entry  (string pronunciation)
   "Add specified pronunciation for current buffer.
 Arguments STRING and PRONUNCIATION specify what is being defined."
@@ -123,7 +138,7 @@ Arguments STRING and PRONUNCIATION specify what is being defined."
    (t                                   ;turn it on
     (setq emacspeak-pronounce-pronunciation-table
           (emacspeak-pronounce-compose-pronunciation-table))))
-  (setf (gethash  (intern string) emacspeak-pronounce-pronunciation-table) pronunciation)
+  (puthash    string pronunciation emacspeak-pronounce-pronunciation-table)
   (message "Added  local pronunciation in buffer %s"
            (buffer-name)))
 
@@ -134,7 +149,7 @@ Arguments STRING and PRONUNCIATION specify what is being defined."
 ;;; parent stored as a property on child symbol.
 ;;; when dictionary composed for a buffer, inherited dictionaries are
 ;;; also looked up.
-
+;;;###autoload
 (defun emacspeak-pronounce-add-super  (parent child)
   "Make CHILD inherit PARENT's pronunciations."
   (let ((orig (get child 'emacspeak-pronounce-supers)))
@@ -153,53 +168,36 @@ Arguments STRING and PRONUNCIATION specify what is being defined."
     orig))
 
 (defun emacspeak-pronounce-compose-pronunciation-table  (&optional buffer)
-  "Composes a pronunciation table for BUFFER.
-The default  is current
-buffer.  Handles inheritance of pronunciation dictionaries between
+  "Composes a pronunciation table for BUFFER. The default is current
+buffer. Handles inheritance of pronunciation dictionaries between
 modes."
   (setq buffer (or buffer (current-buffer )))
-  (let* ((table (make-hash-table))
+  (let* ((table (make-hash-table :test #'equal))
          (filename (buffer-file-name buffer))
-         (directory (and filename
-                         (file-name-directory  filename)))
+         (directory (and filename (file-name-directory  filename)))
          (mode  (save-excursion
                   (set-buffer buffer)
                   major-mode))
          (mode-supers (emacspeak-pronounce-get-supers mode))
-         (file-alist (and filename
-                          (emacspeak-pronounce-get-dictionary filename)))
-         (dir-alist (and directory
-                         (emacspeak-pronounce-get-dictionary directory)))
+         (file-alist (and filename (emacspeak-pronounce-get-dictionary filename)))
+         (dir-alist (and directory (emacspeak-pronounce-get-dictionary directory)))
          (mode-alist (emacspeak-pronounce-get-dictionary mode))
          (super-alist nil))
     (loop for super in mode-supers
           do
-          (setq super-alist
-                (emacspeak-pronounce-get-dictionary super))
+          (setq super-alist (emacspeak-pronounce-get-dictionary super))
           (loop for element in super-alist
                 do
-                (setf (gethash
-                       (intern (car element))
-                       table)
-                      (cdr element )))      )
+                (puthash (car element) (cdr element ) table)))
     (loop for element in mode-alist
           do
-          (setf (gethash
-                 (intern (car element))
-                 table)
-                (cdr element )))
+          (puthash (car element) (cdr element ) table))
     (loop for element in dir-alist
           do
-          (setf (gethash
-                 (intern (car element))
-                 table)
-                (cdr element )))
+          (puthash (car element) (cdr element ) table))
     (loop for element in file-alist
           do
-          (setf (gethash
-                 (intern (car element))
-                 table)
-                (cdr element )))
+          (puthash (car element) (cdr element ) table))
     table))
 
 ;;}}}
@@ -213,7 +211,6 @@ modes."
 ;;; c++ mode inherits from C mode
 (emacspeak-pronounce-add-super  'c-mode 'c++-mode)
 
-
 ;;; latex-mode and latex2e-mode inherit from plain-tex-mode
 
 (emacspeak-pronounce-add-super  'plain-tex-mode 'latex-mode)
@@ -222,7 +219,8 @@ modes."
 (emacspeak-pronounce-add-super 'text-mode 'latex-mode)
 (emacspeak-pronounce-add-super 'text-mode 'latex2e-mode)
 (emacspeak-pronounce-add-super 'text-mode 'plain-tex-mode)
-
+;;;xsl inherits from xml
+(emacspeak-pronounce-add-super 'xml-mode 'xsl-mode)
 ;;}}}
 ;;{{{  Composing and applying dictionaries:
 
@@ -241,44 +239,95 @@ Argument CHILD  specifies the mode whose supers are being requested."
   (get child 'emacspeak-pronounce-supers))
 
 ;;}}}
+
 (defcustom emacspeak-pronounce-pronunciation-personality nil
   "*Pronunciation personality.
 This is the personality used when speaking  things that have a pronunciation
 applied."
-  :group 'emacspeak
-  :type 'symbol)
-
+  :group 'emacspeak-pronounce
+  :type (voice-setup-custom-menu))
+;;;###autoload
 (defsubst emacspeak-pronounce-apply-pronunciations (pronunciation-table )
   "Applies pronunciations specified in pronunciation table to current buffer.
 Modifies text and point in buffer."
-  (loop for  key  being the hash-keys  of pronunciation-table
-        do
-        (let ((word (symbol-name key))
-              (pronunciation (cl-gethash  key pronunciation-table )))
-          (goto-char (point-min))
-          (while (search-forward  word nil t)
-            (replace-match  pronunciation t t  )
-            (and emacspeak-pronounce-pronunciation-personality
-                 (put-text-property
-                  (match-beginning 0)
-                  (+ (match-beginning 0) (length pronunciation))
-                  'personality
-                  emacspeak-pronounce-pronunciation-personality))))))
+  (declare (special emacspeak-pronounce-pronunciation-personality))
+  (let ((words
+         (sort 
+          (loop for  key  being the hash-keys  of pronunciation-table collect key)
+          #'(lambda (a b ) 
+              (> (length  a) (length  b))))))
+    (loop for key in words 
+          do
+          (let ((word  key)
+                (pronunciation (gethash  key pronunciation-table))
+                (pp nil)
+                (personality nil))
+            (when word 
+	      (goto-char (point-min))
+	      (cond
+	       ((stringp pronunciation)
+		(while (search-forward  word nil t)
+		  (setq personality (get-text-property (point) 'personality))
+		  (replace-match  pronunciation t t  )
+		  (put-text-property
+		   (match-beginning 0)
+		   (+ (match-beginning 0) (length pronunciation))
+		   'personality
+		   (apply
+		    'append
+		    (mapcar
+		     #'(lambda (p)
+			 (when p
+			   (if (atom p) (list p) p)))
+		     (list emacspeak-pronounce-pronunciation-personality personality))))))
+	       ((consp pronunciation )
+		(let ((matcher (car pronunciation))
+		      (pronouncer (cdr pronunciation))
+		      (pronunciation ""))
+		  (while (funcall matcher   word nil t)
+		    (setq personality
+			  (get-text-property (point) 'personality))
+		    (setq pronunciation
+			  (save-match-data 
+			    (funcall pronouncer
+				     (buffer-substring 
+				      (match-beginning 0)
+				      (match-end 0)))))
+		    (replace-match pronunciation t t  )
+		    ;; get personality if any from pronunciation
+		    (setq pp
+			  (get-text-property (match-beginning 0) 'personality))
+		    (put-text-property
+		     (match-beginning 0)
+		     (+ (match-beginning 0) (length pronunciation))
+		     'personality
+		     (apply 'append
+			    (mapcar
+			     #'(lambda (p)
+				 (when p
+				   (if (atom p) (list p) p)))
+			     (list
+			      emacspeak-pronounce-pronunciation-personality
+			      personality pp)))))))
+	       (t nil)))))))
 
 ;;}}}
 ;;{{{  loading, clearing  and saving dictionaries
 
-(defcustom emacspeak-pronounce-dictionaries-file  nil
+(declaim (special emacspeak-resource-directory))
+
+(defcustom emacspeak-pronounce-dictionaries-file  
+  (expand-file-name  ".dictionary" 
+                     emacspeak-resource-directory)
   "File that holds the persistent emacspeak pronunciation dictionaries."
   :type '(file :tag "Dictionary File ")
-  :group 'emacspeak)
-
-(declaim (special emacspeak-resource-directory))
-(setq emacspeak-pronounce-dictionaries-file
-      (expand-file-name  ".dictionary" 
-                         emacspeak-resource-directory))
-
+  :group 'emacspeak-pronounce)
 
+(defcustom emacspeak-pronounce-load-pronunciations-on-startup  t
+  "Says if user dictionaries loaded on  emacspeak startup."
+  :type 'boolean
+  :group 'emacspeak-pronounce)
+;;;###autoload
 (defun emacspeak-pronounce-save-dictionaries  ()
   "Writes out the persistent emacspeak pronunciation dictionaries."
   (interactive)
@@ -291,6 +340,7 @@ Modifies text and point in buffer."
     (setq buffer (find-file-noselect filename))
     (save-excursion
       (set-buffer buffer)
+      (auto-fill-mode nil)
       (erase-buffer)
       (loop for key being the hash-keys  of emacspeak-pronounce-dictionaries
             do
@@ -298,13 +348,12 @@ Modifies text and point in buffer."
              (format "(emacspeak-pronounce-set-dictionary '%S\n '%S )\n"
                      key
                      (emacspeak-pronounce-get-dictionary key ))))
-      (goto-char (point-min))
-      (while (search-forward ")" nil t)
-        (replace-match ")\n" nil t))
       (save-buffer))))
+
+;;;###autoload
 (defvar emacspeak-pronounce-dictionaries-loaded nil
   "Indicates if dictionaries already loaded.")
-
+;;;###autoload
 (defun emacspeak-pronounce-load-dictionaries  (&optional filename)
   "Load pronunciation dictionaries.
 Optional argument FILENAME specifies the dictionary file."
@@ -314,9 +363,10 @@ Optional argument FILENAME specifies the dictionary file."
           emacspeak-resource-directory
           emacspeak-pronounce-dictionaries-file )))
   (declare (special emacspeak-pronounce-dictionaries-loaded))
-  (load-file filename)
-  (setq emacspeak-pronounce-dictionaries-loaded t))
-
+  (when (file-exists-p filename)
+    (load-file filename)
+    (setq emacspeak-pronounce-dictionaries-loaded t)))
+;;;###autoload
 (defun emacspeak-pronounce-clear-dictionaries ()
   "Clear all current pronunciation dictionaries."
   (interactive)
@@ -360,6 +410,11 @@ Optional argument FILENAME specifies the dictionary file."
     (insert string)
     (dtk-speak string)))
 
+(defsubst emacspeak-pronounce-read-pattern (key)
+  (declare (special emacspeak-pronounce-yank-word-point
+                    emacspeak-pronounce-current-buffer))
+  (eval (read-minibuffer "Pattern")))
+
 (defsubst emacspeak-pronounce-read-term (key)
   (declare (special emacspeak-pronounce-yank-word-point
                     emacspeak-pronounce-current-buffer))
@@ -379,7 +434,7 @@ Optional argument FILENAME specifies the dictionary file."
        now-map))))
 
 (defun emacspeak-pronounce-define-local-pronunciation (word  pronunciation )
-  "Define buffer local pronounciation.
+  "Define buffer local pronunciation.
 Argument WORD specifies the word which should be pronounced as specified by PRONUNCIATION."
   (interactive
    (list
@@ -389,17 +444,13 @@ Argument WORD specifies the word which should be pronounced as specified by PRON
   (emacspeak-pronounce-add-buffer-local-dictionary-entry
    word pronunciation))
 
-(defun emacspeak-pronounce-define-pronunciation ()
-  "Interactively define entries in the pronunciation dictionaries.
-Default term to define is delimited by region.
-First loads any persistent dictionaries if not already loaded."
-  (interactive)
-  (declare (special emacspeak-pronounce-dictionaries-loaded))
+(defsubst emacspeak-pronounce-get-key ()
+  "Collect key from user.
+Returns a pair of the form (key-type . key)."
+  (declare (special emacspeak-pronounce-pronunciation-keys))
   (let ((key nil)
-        (word nil)
-        (pronunciation nil)
-        (key-type (completing-read  "Define pronunciation that is specific to: "
-                                    emacspeak-pronounce-pronunciation-keys nil t )))
+        (key-type
+         (completing-read  "Define pronunciation that is specific to: " emacspeak-pronounce-pronunciation-keys nil t )))
     (cond
      ((string= key-type "buffer")
       (setq key (buffer-name )))        ;handled differently
@@ -422,32 +473,67 @@ First loads any persistent dictionaries if not already loaded."
             major-mode)
       (or key (error "No major mode found for current buffer")))
      (t (error "Cannot define pronunciations with key type %s" key-type)))
-    (setq word (emacspeak-pronounce-read-term key))
+    (cons key-type key)))
+
+(defun emacspeak-pronounce-define-template-pronunciation ()
+  "Interactively define template entries in the pronunciation dictionaries.
+Default term to define is delimited by region.
+First loads any persistent dictionaries if not already loaded."
+  (interactive)
+  (declare (special emacspeak-pronounce-dictionaries-loaded))
+  (let ((word nil)
+        (pronunciation nil)
+        (key-pair (emacspeak-pronounce-get-key)))
+    (setq word (emacspeak-pronounce-read-pattern (cdr key-pair)))
+    (setq pronunciation
+          (cons
+           (read-minibuffer
+            (format "Matcher for  %s: " word))
+           (read-minibuffer
+            (format "Pronouncer for  %s: " word))))
+    (when (and (not emacspeak-pronounce-dictionaries-loaded)
+               (y-or-n-p "Load pre existing  pronunciation dictionaries first? "))
+      (emacspeak-pronounce-load-dictionaries))
+    (unless  (string= (car key-pair)  "buffer")
+      (emacspeak-pronounce-add-dictionary-entry (cdr key-pair) word pronunciation)
+      (emacspeak-pronounce-refresh-pronunciations))
+    (when (string= (car key-pair)  "buffer")
+      (emacspeak-pronounce-add-buffer-local-dictionary-entry word pronunciation))))
+
+(defun emacspeak-pronounce-define-pronunciation ()
+  "Interactively define entries in the pronunciation dictionaries.
+Default term to define is delimited by region.
+First loads any persistent dictionaries if not already loaded."
+  (interactive)
+  (declare (special emacspeak-pronounce-dictionaries-loaded))
+  (let ((word nil)
+        (pronunciation nil)
+        (key-pair(emacspeak-pronounce-get-key)))
+    (setq word (emacspeak-pronounce-read-term (cdr key-pair)))
     (setq pronunciation
           (read-from-minibuffer
            (format "Pronounce %s as: " word)))
     (when (and (not emacspeak-pronounce-dictionaries-loaded)
                (y-or-n-p "Load pre existing  pronunciation dictionaries first? "))
       (emacspeak-pronounce-load-dictionaries))
-    (unless  (string= key-type  "buffer")
-      (emacspeak-pronounce-add-dictionary-entry key word pronunciation)
+    (unless  (string= (car key-pair)  "buffer")
+      (emacspeak-pronounce-add-dictionary-entry (cdr key-pair) word pronunciation)
       (emacspeak-pronounce-refresh-pronunciations))
-    (when (string= key-type  "buffer")
+    (when (string= (car key-pair)  "buffer")
       (emacspeak-pronounce-add-buffer-local-dictionary-entry  word pronunciation))))
 
-
 ;;}}}
 ;;{{{ Turning dictionaries on and off on a per buffer basis
-
+;;;###autoload
 (defvar emacspeak-pronounce-pronunciation-table nil
   "Variable holding association list of pronunciations for a buffer.
 Becomes automatically buffer local.")
 (make-variable-buffer-local 'emacspeak-pronounce-pronunciation-table)
 (setq-default emacspeak-pronounce-pronunciation-table nil)
-
+;;;###autoload
 (defun emacspeak-pronounce-toggle-use-of-dictionaries (&optional state)
   "Toggle use of pronunciation dictionaries in current buffer.
-Pronunciations can be dfined on a per file, per directory and/or per
+Pronunciations can be defined on a per file, per directory and/or per
 mode basis.
 Pronunciations are activated on a per buffer basis.
 Turning on the use of pronunciation dictionaries results in emacspeak
@@ -457,7 +543,7 @@ After this, the pronunciations will be applied whenever text in the
 buffer is spoken.
 Optional argument state can be used from Lisp programs to
 explicitly turn pronunciations on or off."
-  (interactive)
+  (interactive "P")
   (declare (special emacspeak-pronounce-pronunciation-table))
   (cond
    ((or (eq state 'on)
@@ -480,7 +566,7 @@ explicitly turn pronunciations on or off."
     (message
      "Emacspeak pronunciations have been re-activated in this buffer")
     (emacspeak-auditory-icon 'on))))
-
+;;;###autoload
 (defun emacspeak-pronounce-refresh-pronunciations ()
   "Refresh pronunciation table for current buffer.
 Activates pronunciation dictionaries if not already active."
@@ -490,27 +576,85 @@ Activates pronunciation dictionaries if not already active."
    ((not (boundp 'emacspeak-pronounce-pronunciation-table)) ;first time
     (set (make-local-variable 'emacspeak-pronounce-pronunciation-table)
          (emacspeak-pronounce-compose-pronunciation-table))
-    (emacspeak-auditory-icon 'on)
-    (message
-     "Refreshed pronunciations for this buffer"))
+    )
    ( emacspeak-pronounce-pronunciation-table ;already on --refresh it
      (setq emacspeak-pronounce-pronunciation-table
            (emacspeak-pronounce-compose-pronunciation-table))
-     (emacspeak-auditory-icon 'on)
-     (message
-      "Refreshed pronunciation for this buffer"))
+     )
    (t                                   ;turn it on
     (setq emacspeak-pronounce-pronunciation-table
-          (emacspeak-pronounce-compose-pronunciation-table))
+          (emacspeak-pronounce-compose-pronunciation-table))))
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'on)
     (message
-     "Refreshed pronunciations for this buffer")
-    (emacspeak-auditory-icon 'on))))
+     "Refreshed pronunciations for this buffer")))
+
+;;}}}
+;;{{{ common dictionary containing smileys and friends
+
+(defcustom emacspeak-pronounce-internet-smileys-pronunciations 
+  '((":-)" . " smile ")
+    (":)" . " grin ")
+    (":-(" . " frown ") 
+    (":(" . " sigh ")
+    (":-I" .  " shrug ")
+    (":->" . " sarcastic smile ")
+    (">:->" . " devillish smile ")
+    (">;->" . " lews smile ")
+    (";-)" . " wink "))
+  "Pronunciation dictionary used in all instant messenger and IRC chat
+modes.
+See http://oz.uc.edu/~solkode/smileys.html for a full list."
+  :link '(url-link :tag "Smileys Dictionary "
+		   "http://oz.uc.edu/~solkode/smileys.html")
+  :type '(repeat
+	  (cons :tag "Dictionary Entry"
+		(string :tag "String")
+		(string :tag "Pronunciation")))
+  :group 'emacspeak-pronounce )
+
+;;}}}
+;;{{{  xml namespace uri's
+
+(defcustom emacspeak-pronounce-common-xml-namespace-uri-pronunciations 
+  '(
+    ("http://www.w3.org/1999/02/22-rdf-syntax-ns#" . "RDF Syntax")
+    ("http://www.w3.org/2002/06/xhtml2" . " xhtml2 ")
+    ("http://www.w3.org/1999/XSL/Transform" . " XSLT ")
+    ("http://www.w3.org/2002/xforms" . " XForms ")
+    ("http://www.w3.org/2001/xml-events" . " XEvents ")
+    ("http://www.w3.org/2001/vxml" . " vxml ")
+    ("http://www.w3.org/2001/XMLSchema-instance". " XSchema Instance ")
+    ("http://www.w3.org/2001/XMLSchema". " XSchema ")
+    ("http://www.w3.org/1999/xhtml" . " xhtml ")
+    ("http://schemas.xmlsoap.org/wsdl/"  . " WSDL ")
+    ("http://www.w3.org/2001/06/soap-envelope" . " SOAP ENV  ")
+    ("http://schemas.xmlsoap.org/wsdl/soap/"  . " SOAP ")
+    )
+  "Pronunciations for well known namespace URIs."
+  :type '(repeat
+	  (cons :tag "Dictionary Entry"
+		(string :tag "Namespace URI")
+		(string :tag "Pronunciation")))
+  :group 'emacspeak-pronounce )
+
+;;}}}
+;;{{{ adding predefined dictionaries to a mode:
+;;;###autoload
+(defun emacspeak-pronounce-augment-pronunciations (mode dictionary)
+  "Pushes pronunciations in specified dictionary on to the dictionary
+for the specified mode."
+  (let ((mode-alist (emacspeak-pronounce-get-dictionary mode)))
+    (loop for e in dictionary
+          do
+          (unless (assoc (car e)
+                         mode-alist)
+	    (push e mode-alist)))
+    (emacspeak-pronounce-set-dictionary mode mode-alist)))
 
 ;;}}}
 ;;{{{  dictionary editor 
 
-
-
 (defun emacspeak-pronounce-edit-generate-pronunciation-editor  (key)
   "Generate a widget-enabled edit buffer for editting the
 pronunciation dictionary for the specified key."
@@ -537,7 +681,11 @@ pronunciation dictionary for the specified key."
                      :notify notify
                      '(cons :tag "Dictionary Entry"
                             (string :tag "Phrase")
-                            (string :tag "Pronounce as")))
+                            (choice  :tag "Pronunciation"
+                                     (string :tag "Pronounce as")
+                                     (cons :tag "Pronouncer"
+                                           (symbol :tag "Matcher")
+                                           (symbol :tag "Pronouncer")))))
       (widget-insert "\n")
       (widget-create 'push-button
                      :tag "Save Dictionary"
@@ -563,28 +711,28 @@ pronunciation dictionary for the specified key."
         value)))))
 
 (defun emacspeak-pronounce-edit-pronunciations (key)
-   "Prompt for and launch a pronunciation editor on the
+  "Prompt for and launch a pronunciation editor on the
 specified pronunciation dictionary key."
-   (interactive
-    (list
-     (let ((keys
-            (loop for k being the hash-keys of
-                  emacspeak-pronounce-dictionaries
-                  collect
-                  (symbol-name k))))
-       (completing-read "Edit dictionary: "
-                        (mapcar
-                         #'(lambda (k)
-                             (cons k k ))
-                         keys)
-                        nil
-                        'REQUIRE-MATCH 
-                        nil
-                        'keys
-                        (car keys))))) 
-   (declare (special emacspeak-pronounce-dictionaries))
-   (emacspeak-pronounce-edit-generate-pronunciation-editor
-    (intern key)))
+  (interactive
+   (list
+    (let ((keys
+	   (loop for k being the hash-keys of
+		 emacspeak-pronounce-dictionaries
+		 collect
+		 (symbol-name k))))
+      (completing-read "Edit dictionary: "
+		       (mapcar
+			#'(lambda (k)
+			    (cons k k ))
+			keys)
+		       nil
+		       'REQUIRE-MATCH 
+		       nil
+		       'keys
+		       (car keys))))) 
+  (declare (special emacspeak-pronounce-dictionaries))
+  (emacspeak-pronounce-edit-generate-pronunciation-editor
+   (intern key)))
 
 ;;}}}
 ;;{{{ top level dispatch routine
@@ -603,6 +751,7 @@ specified pronunciation dictionary key."
       (?c (call-interactively 'emacspeak-pronounce-clear-dictionaries))
       (?d (call-interactively
            'emacspeak-pronounce-define-pronunciation t))
+      (?D (call-interactively 'emacspeak-pronounce-define-template-pronunciation t))
       (?e (call-interactively
            'emacspeak-pronounce-edit-pronunciations t))
       (?l (call-interactively 'emacspeak-pronounce-load-dictionaries))
@@ -612,6 +761,61 @@ specified pronunciation dictionary key."
       (otherwise (message emacspeak-pronounce-help)))
     (emacspeak-auditory-icon 'close-object)))
 
+;;}}}
+;;{{{ Helpers: pronouncers
+
+;;{{{ dates
+
+(defvar emacspeak-pronounce-date-mm-dd-yyyy-pattern
+  "[0-9]\\{2\\}-[0-9]\\{2\\}-[0-9]\\{2\\}\\([0-9]\\{2\\}\\)?"
+  "Pattern that matches dates of the form mm-dd-[cc]yy.")
+
+(defun emacspeak-pronounce-mm-dd-yyyy-date (string)
+  "Return pronunciation for mm-dd-yyyy dates."
+  (save-match-data
+    (let ((fields (mapcar #'read (split-string string "-"))))
+      (propertize
+       (calendar-date-string
+        (list (second fields)
+              (first fields)
+              (cond
+               ((< (third fields) 50)
+                (+ 2000 (third fields)))
+               ((< (third fields) 100)
+                (+ 1900 (third fields)))
+               (t (third fields)))))
+       'personality voice-punctuations-some))))
+                   
+;;}}}
+;;{{{ phone numbers
+
+(defvar emacspeak-pronounce-us-phone-number-pattern "1?[0-9]\\{3\\}-[0-9]\\{3}-[0-9]\\{4\\}"
+  "Pattern that matches US phone numbers.")
+
+(defun emacspeak-pronounce-us-phone-number (phone)
+  "Return pronunciation for US phone number."
+  (when (= 14 (length phone))
+    (setq phone (substring phone 2)))
+  (let ((area-code (substring  phone 0 3))
+        (prefix-code  (substring  phone 4 7))
+        (suffix-code  (substring  phone 8 12)))
+    (unless (string-equal "800" area-code)
+      (setq area-code
+            (replace-regexp-in-string
+             "[0-9]" " \\&" area-code)))
+    (setq prefix-code
+          (replace-regexp-in-string
+           "[0-9]" " \\&" prefix-code))
+    (setq suffix-code
+          (replace-regexp-in-string
+           "[0-9]\\{2\\}" " \\&"  suffix-code))
+    (propertize 
+     (format "(%s) %s, %s. "
+             area-code prefix-code suffix-code)
+     'personality voice-punctuations-some)))
+
+;;}}}
+
 ;;}}}
 
 (provide  'emacspeak-pronounce)
@@ -619,7 +823,7 @@ specified pronunciation dictionary key."
 
 ;;; local variables:
 ;;; folded-file: t
-;;; byte-compile-dynamic: nil
+;;; byte-compile-dynamic: t
 ;;; end:
 
 ;;}}}
