@@ -1,5 +1,5 @@
 ;;; emacspeak-vm.el --- Speech enable VM -- A powerful mail agent (and the one I use)
-;;; $Id: emacspeak-vm.el,v 16.0 2002/05/03 23:31:24 raman Exp $
+;;; $Id: emacspeak-vm.el,v 20.0 2004/05/01 01:16:23 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak extension to speech enhance vm
 ;;; Keywords: Emacspeak, VM, Email, Spoken Output, Voice annotations
@@ -8,15 +8,15 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@cs.cornell.edu 
 ;;; A speech interface to Emacs |
-;;; $Date: 2002/05/03 23:31:24 $ |
-;;;  $Revision: 16.0 $ | 
+;;; $Date: 2004/05/01 01:16:23 $ |
+;;;  $Revision: 20.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
 
-;;;Copyright (C) 1995 -- 2002, T. V. Raman 
+;;;Copyright (C) 1995 -- 2003, T. V. Raman 
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -37,94 +37,43 @@
 ;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;}}}
-(require 'cl)
-(declaim  (optimize  (safety 0) (speed 3)))
-(require 'custom)
-(require 'dtk-voices)
-(require 'voice-lock)
-(require 'emacspeak-keymap)
-(require 'dtk-speak)
-(require 'emacspeak-sounds)
-(require 'emacspeak-speak)
+
 ;;{{{  Introduction:
 ;;; This module extends the mail reader vm.
 ;;; Uses voice locking for message headers and cited messages
 
+;;}}}
+;;{{{ requires
+(require 'emacspeak-preamble)
+
 ;;}}}
 ;;{{{ voice locking:
 
-(defgroup  emacspeak-vm nil
+(defgroup emacspeak-vm nil
   "VM mail reader on the Emacspeak Desktop."
   :group 'emacspeak
   :group 'vm
   :prefix "emacspeak-vm-")
 
-
 (defcustom emacspeak-vm-voice-lock-messages nil
   "Set this to T if you want messages automatically voice locked.
 Note that some badly formed mime messages  cause trouble."
   :type 'boolean
   :group 'emacspeak-vm)
 
-(defvar vm-voice-lock-keywords nil
-  "Keywords to highlight in vm")
-
-(defvar vm-summary-voice-lock-keywords
-  "Additional expressions to highlight in vm  Summary mode.")
-
-;;;  Set vm-voice-lock-keywords
-
-(setq vm-voice-lock-keywords
-      (append vm-voice-lock-keywords
-              '(("^From: \\(.*\\)$" 1  emacspeak-vm-from-voice )
-                ("^To: \\(.*\\)$" 1 emacspeak-vm-to-voice)
-                ("^Subject: \\(.*\\)$" 1 emacspeak-vm-subject-voice)
-                ("^|?[a-zA-Z]*>+\\(.*\\)$" 1 emacspeak-vm-cite-voice )
-                )))
-
-(voice-lock-set-major-mode-keywords 'vm-mode      'vm-voice-lock-keywords)
-
-(voice-lock-set-major-mode-keywords 'vm-summary-mode
-                                    'vm-summary-voice-lock-keywords)
-
 (add-hook 'vm-mode-hook
           'emacspeak-vm-mode-setup)
 
 (defun emacspeak-vm-mode-setup ()
   "Setup function placed on vm-mode-hook by Emacspeak."
-  (declare (special  dtk-punctuation-mode
-                     dtk-allcaps-beep
-                     emacspeak-vm-voice-lock-messages))
+  (declare (special  dtk-punctuation-mode emacspeak-vm-voice-lock-messages
+                     dtk-allcaps-beep))
   (setq dtk-punctuation-mode "some")
   (when dtk-allcaps-beep
     (dtk-toggle-allcaps-beep))
   (emacspeak-dtk-sync)
   (when emacspeak-vm-voice-lock-messages
-    (condition-case nil
-        (voice-lock-mode 1)
-      (error nil ))))
-
-;;}}}
-;;{{{  vm voices:
-
-(defcustom emacspeak-vm-from-voice  'harry
-  "Personality for From field. "
-  :type 'symbol
-  :group 'emacspeak-vm)
-
-(defcustom emacspeak-vm-to-voice  'paul-animated
-  "Personality for To field. "
-  :type 'symbol
-  :group 'emacspeak-vm)
-(defcustom emacspeak-vm-subject-voice  'paul-smooth 
-  "Personality for Subject field. "
-  :type 'symbol
-  :group 'emacspeak-vm)
-
-(defcustom emacspeak-vm-cite-voice  'paul-smooth
-  "Personality for citation lines. "
-  :type 'symbol
-  :group 'emacspeak-vm)
+    (voice-lock-mode 1)))
 
 ;;}}}
 ;;{{{ inline helpers
@@ -175,6 +124,28 @@ Note that some badly formed mime messages  cause trouble."
 
 (defvar emacspeak-vm-user-login-name  (user-login-name)
   "Login name of this user")
+(defun emacspeak-vm-yank-header ()
+  "Yank specified header into kill ring."
+  (interactive)
+  (declare (special vm-message-pointer))
+  (cond
+   (vm-message-pointer
+    (dtk-stop)
+    (let*  ((message (car vm-message-pointer ))
+            (from (vm-su-from message ))
+            (subject (vm-so-sortable-subject message ))
+            (to (vm-su-to message ))
+            (header nil))
+      (while (not header)
+	(setq header 
+	      (case
+		  (read-char "f From s Subject t To")
+		(?s subject)
+		(?f from)
+		(?t to))))
+      (kill-new header)
+      (message header)))
+   (t (error "No current message." ))))
 
 (defun emacspeak-vm-summarize-message ()
   "Summarize the current vm message. "
@@ -208,7 +179,7 @@ Note that some badly formed mime messages  cause trouble."
         (emacspeak-auditory-icon 'item))
        (self-p                          ;mail to others including me
         (emacspeak-auditory-icon 'mark-object))
-       (t                               ;got it because of a mailing list
+       (t			     ;got it because of a mailing list
         (emacspeak-auditory-icon 'select-object ))))))
 
 (defun emacspeak-vm-speak-labels ()
@@ -273,7 +244,7 @@ Note that some badly formed mime messages  cause trouble."
 Useful when you're reading a message
 that has been forwarded multiple times."
   (interactive)
-  (re-search-forward "^Subject:" nil t )
+  (re-search-forward "^ *Subject:" nil t )
   (emacspeak-speak-line))
 
 (defadvice vm-scroll-forward (after emacspeak pre act)
@@ -310,6 +281,7 @@ Then speak the screenful. "
 
 (declaim (special vm-mode-map))
 (define-key vm-mode-map "\M-\C-m" 'widget-button-press)
+(define-key vm-mode-map "y" 'emacspeak-vm-yank-header)
 (define-key vm-mode-map "\M-\t"
   'emacspeak-vm-next-button)
 (define-key vm-mode-map  "j"
@@ -394,18 +366,31 @@ Then speak the screenful. "
 ;;}}}
 ;;{{{ quitting
 
-
 (defadvice vm-quit (after emacspeak pre act )
   "Provide an auditory icon if requested"
   (when (interactive-p)
     (emacspeak-auditory-icon 'close-object)))
 
+;;}}}
+;;{{{ catching up on folders
+
+(defun emacspeak-vm-catch-up-all-messages ()
+  "Mark all messages in folder to be deleted. Use with caution."
+  (interactive)
+  (declare (special vm-ml-highest-message-number))
+  (vm-goto-message 1)
+  (vm-delete-message
+   (read vm-ml-highest-message-number))
+  (message "All messages have been marked as deleted.")
+  (emacspeak-auditory-icon 'delete-object))
+
 ;;}}}
 ;;{{{  Keybindings:
 (declaim  (special vm-mode-map
                    global-map
                    emacspeak-prefix
                    emacspeak-keymap))
+(define-key vm-mode-map "C" 'emacspeak-vm-catch-up-all-messages)
 (define-key vm-mode-map "\M-j" 'emacspeak-vm-locate-subject-line)
 (define-key vm-mode-map "\M-l" 'emacspeak-vm-speak-labels)
 (define-key vm-mode-map
@@ -428,7 +413,6 @@ Then speak the screenful. "
    (t ad-do-it))
   ad-return-value)
 
-
 (defadvice vm-isearch-backward (around emacspeak pre act comp)
   "Provide auditory feedback"
   (declare (special vm-message-pointer))
@@ -462,7 +446,6 @@ Leave point at front of decoded attachment."
    ((interactive-p)
     (let ((orig (point )))
       ad-do-it
-      (emacspeak-auditory-icon 'task-done)
       (goto-char orig)
       (message "Decoded attachment")))
    (t ad-do-it))
@@ -493,7 +476,25 @@ Leave point at front of decoded attachment."
 (add-hook 'vm-presentation-mode-hook
           (function
            (lambda nil
-             (modify-syntax-entry 10 " "))))
+             (emacspeak-pronounce-refresh-pronunciations))))
+(declaim (special emacspeak-pronounce-internet-smileys-pronunciations))
+(loop for hook in 
+      (list 'mail-mode-hook
+            'vm-presentation-mode-hook)
+      do 
+      (add-hook hook 'emacspeak-pronounce-refresh-pronunciations
+                'append ))
+
+(loop for mode in 
+      '(vm-presentation-mode 
+        mail-mode)
+      do 
+      (emacspeak-pronounce-augment-pronunciations mode
+                                                  emacspeak-pronounce-internet-smileys-pronunciations)
+      (emacspeak-pronounce-add-dictionary-entry mode
+                                                emacspeak-speak-iso-datetime-pattern
+                                                (cons 're-search-forward
+                                                      'emacspeak-speak-decode-iso-datetime)))          
 
 ;;}}}
 ;;{{{ advice button motion 
@@ -503,7 +504,6 @@ Leave point at front of decoded attachment."
     (emacspeak-auditory-icon 'large-movement)
     (emacspeak-speak-text-range  'w3-hyperlink-info)))
 
-
 ;;}}}
 ;;{{{  misc 
 
@@ -553,18 +553,21 @@ If N is negative, move backward instead."
 (defcustom emacspeak-vm-use-raman-settings t
   "Should VM  use the customizations used by the author of Emacspeak."
   :type 'boolean
-  :group 'emacspeak)
-
-
-
+  :group 'emacspeak-vm)
 
 (defun emacspeak-vm-use-raman-settings ()
   "Customization settings for VM used by the author of
 Emacspeak."
-  (declare (special 
+  (declare (special
+            vm-mime-charset-converter-alist
+            vm-mime-default-face-charsets 
+            vm-frame-per-folder 
+            vm-frame-per-composition 
+            vm-frame-per-edit 
+            vm-frame-per-help 
+            vm-frame-per-summary 
             vm-index-file-suffix
             vm-primary-inbox
-            vm-keep-sent-messages
             vm-folder-directory
             vm-forwarding-subject-format
             vm-startup-with-summary
@@ -575,9 +578,20 @@ Emacspeak."
             vm-confirm-new-folders
             vm-move-after-deleting
             emacspeak-vm-voice-lock-messages))
+  (setq vm-mime-charset-converter-alist
+	'(
+	  ("utf-8" "iso-8859-1" "iconv -f utf-8 -t iso-8859-1")
+	  )
+	)
+  (setq vm-mime-default-face-charsets t)
+  (setq vm-frame-per-folder nil
+	vm-frame-per-composition nil
+	vm-frame-per-edit nil
+	vm-frame-per-help nil
+	vm-frame-per-summary nil)
+
   (setq vm-index-file-suffix ".idx"
         vm-primary-inbox "~/mbox"
-        vm-keep-sent-messages t
         vm-folder-directory "~/Mail/"
         vm-forwarding-subject-format "[%s]"
         vm-startup-with-summary nil
@@ -587,7 +601,7 @@ Emacspeak."
         vm-url-browser 'browse-url
         vm-confirm-new-folders t
         vm-move-after-deleting nil
-        emacspeak-vm-voice-lock-messages nil)
+        emacspeak-vm-voice-lock-messages t)
   t)
   
 (when emacspeak-vm-use-raman-settings
@@ -612,7 +626,12 @@ text using pdftotext."
 text using wvText."
   :type 'string
   :group 'emacspeak-vm)
-
+(defcustom emacspeak-vm-cal2text
+  (expand-file-name "cal2text" emacspeak-etc-directory)
+  "Executable that converts calendar invitations    on
+  standard input to plain text."
+  :type 'string
+  :group 'emacspeak-vm)
 
 (defcustom emacspeak-vm-xls2html
   (expand-file-name "xls2html" emacspeak-etc-directory)
@@ -628,7 +647,6 @@ text using wvText."
   :type 'string
   :group 'emacspeak-vm)
 
-
 (defsubst emacspeak-vm-add-mime-convertor (convertor)
   "Helper to add a convertor specification."
   (declare (special vm-mime-type-converter-alist))
@@ -642,21 +660,23 @@ text using wvText."
 
 (defun emacspeak-vm-customize-mime-settings ()
   "Customize VM mime settings."
-  (declare (special
-            vm-preview-lines
-            vm-infer-mime-types
-            vm-mime-decode-for-preview
-            vm-auto-decode-mime-messages
-            vm-auto-displayed-mime-content-type-exceptions
-            vm-mime-attachment-save-directory
-            vm-mime-base64-encoder-program
-            vm-mime-base64-decoder-program
-            vm-mime-attachment-auto-type-alist
-            vm-mime-type-converter-alist
-            emacspeak-vm-pdf2text
-            emacspeak-vm-ppt2html
-            emacspeak-vm-xls2html
-            emacspeak-vm-doc2text))
+  (declare (special vm-preview-lines
+		    vm-infer-mime-types
+		    vm-mime-decode-for-preview
+		    vm-auto-decode-mime-messages
+		    vm-auto-displayed-mime-content-type-exceptions
+		    vm-mime-attachment-save-directory
+		    vm-mime-base64-encoder-program
+		    vm-mime-base64-decoder-program
+		    vm-mime-attachment-auto-type-alist
+		    vm-mime-type-converter-alist
+		    emacspeak-vm-pdf2text
+		    emacspeak-vm-ppt2html
+		    emacspeak-vm-xls2html
+		    emacspeak-vm-doc2text
+		    emacspeak-vm-cal2text))
+  (emacspeak-vm-add-mime-convertor
+   (list "text/calendar" "text/plain" emacspeak-vm-cal2text))
   (emacspeak-vm-add-mime-convertor
    (list "application/pdf" "text/plain"
          emacspeak-vm-pdf2text))
